/*************** <lalVerbatim file="VectorFactoriesTestCV"> ****
$Id: VectorFactoriesTest.m4,v 1.2 2000/06/03 21:36:47 jolien Exp $
**************** </lalVerbatim> *******************************/

/* <lalLaTeX>

\subsection{Program \texttt{VectorFactoriesTest.c}}
\label{ss:VectorFactoriesTest.c}

A program to test create/destroy vector routines.

\subsubsection*{Usage}
\begin{verbatim}
VectorFactoriesTest [options]
Options:
  -h         print help
  -q         quiet: run silently
  -v         verbose: print extra information
  -d level   set debuglevel to level
\end{verbatim}

\subsubsection*{Description}

\subsubsection*{Exit codes}
\begin{tabular}{|c|l|}
\hline
 Code & Explanation                   \\
\hline
\tt 0 & Success, normal exit.         \\
\tt 1 & Subroutine failed.            \\
\hline
\end{tabular}

\subsubsection*{Algorithm}

\subsubsection*{Uses}
\begin{verbatim}
debuglevel
TYPECODECreateVector()
TYPECODEDestroyVector()
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{VectorFactoriesTestCV}}

</lalLaTeX> */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "LALConfig.h"

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include "LALStdlib.h"
#include "AVFactories.h"

#define CODES_(x) #x
#define CODES(x) CODES_(x)

NRCSID( MAIN, "$Id: VectorFactoriesTest.m4,v 1.2 2000/06/03 21:36:47 jolien Exp $" );

extern char *optarg;
extern int   optind;

int debuglevel = 1;
int verbose    = 0;

static void
Usage (const char *program, int exitflag);

static void
ParseOptions (int argc, char *argv[]);

static void
TestStatus (Status *status, const char *expectedCodes, int exitCode);

static void
ClearStatus (Status *status);




















void ZVectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static Status  status;
  static COMPLEX16Vector  *vector;
  static COMPLEX16Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  ZCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( vector->data, 0, length*sizeof( COMPLEX16 ) );

  ZDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  ZCreateVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  ZDestroyVector ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  ZCreateVector ( &status, NULL, length );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  ZDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  vector = &vstore;
  ZCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  ZDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of ZCreateVector and ZDestroyVector \n" );
          
  return;
}





















void CVectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static Status  status;
  static COMPLEX8Vector  *vector;
  static COMPLEX8Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  CCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( vector->data, 0, length*sizeof( COMPLEX8 ) );

  CDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  CCreateVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  CDestroyVector ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  CCreateVector ( &status, NULL, length );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  CDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  vector = &vstore;
  CCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  CDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of CCreateVector and CDestroyVector \n" );
          
  return;
}





















void DVectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static Status  status;
  static REAL8Vector  *vector;
  static REAL8Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  DCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( vector->data, 0, length*sizeof( REAL8 ) );

  DDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  DCreateVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  DDestroyVector ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  DCreateVector ( &status, NULL, length );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  DDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  vector = &vstore;
  DCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  DDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of DCreateVector and DDestroyVector \n" );
          
  return;
}





















void SVectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static Status  status;
  static REAL4Vector  *vector;
  static REAL4Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  SCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( vector->data, 0, length*sizeof( REAL4 ) );

  SDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  SCreateVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  SDestroyVector ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  SCreateVector ( &status, NULL, length );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  SDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  vector = &vstore;
  SCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  SDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of SCreateVector and SDestroyVector \n" );
          
  return;
}





















void I2VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static Status  status;
  static INT2Vector  *vector;
  static INT2Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  I2CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( vector->data, 0, length*sizeof( INT2 ) );

  I2DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  I2CreateVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  I2DestroyVector ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  I2CreateVector ( &status, NULL, length );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  I2DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  vector = &vstore;
  I2CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  I2DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of I2CreateVector and I2DestroyVector \n" );
          
  return;
}





















void I4VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static Status  status;
  static INT4Vector  *vector;
  static INT4Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  I4CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( vector->data, 0, length*sizeof( INT4 ) );

  I4DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  I4CreateVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  I4DestroyVector ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  I4CreateVector ( &status, NULL, length );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  I4DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  vector = &vstore;
  I4CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  I4DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of I4CreateVector and I4DestroyVector \n" );
          
  return;
}





















void I8VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static Status  status;
  static INT8Vector  *vector;
  static INT8Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  I8CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( vector->data, 0, length*sizeof( INT8 ) );

  I8DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  I8CreateVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  I8DestroyVector ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  I8CreateVector ( &status, NULL, length );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  I8DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  vector = &vstore;
  I8CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  I8DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of I8CreateVector and I8DestroyVector \n" );
          
  return;
}





















void U2VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static Status  status;
  static UINT2Vector  *vector;
  static UINT2Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  U2CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( vector->data, 0, length*sizeof( UINT2 ) );

  U2DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  U2CreateVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  U2DestroyVector ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  U2CreateVector ( &status, NULL, length );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  U2DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  vector = &vstore;
  U2CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  U2DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of U2CreateVector and U2DestroyVector \n" );
          
  return;
}





















void U4VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static Status  status;
  static UINT4Vector  *vector;
  static UINT4Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  U4CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( vector->data, 0, length*sizeof( UINT4 ) );

  U4DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  U4CreateVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  U4DestroyVector ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  U4CreateVector ( &status, NULL, length );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  U4DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  vector = &vstore;
  U4CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  U4DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of U4CreateVector and U4DestroyVector \n" );
          
  return;
}





















void U8VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static Status  status;
  static UINT8Vector  *vector;
  static UINT8Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  U8CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( vector->data, 0, length*sizeof( UINT8 ) );

  U8DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  U8CreateVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  U8DestroyVector ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  U8CreateVector ( &status, NULL, length );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  U8DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  vector = &vstore;
  U8CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  U8DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of U8CreateVector and U8DestroyVector \n" );
          
  return;
}





















void CHARVectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static Status  status;
  static CHARVector  *vector;
  static CHARVector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  CHARCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( vector->data, 0, length*sizeof( CHAR ) );

  CHARDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  CHARCreateVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  CHARDestroyVector ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  CHARCreateVector ( &status, NULL, length );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  CHARDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  vector = &vstore;
  CHARCreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  CHARDestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of CHARCreateVector and CHARDestroyVector \n" );
          
  return;
}





















void VectorFactoriesTest ( void )
{
  const  UINT4   length = 16;
  static Status  status;
  static REAL4Vector  *vector;
  static REAL4Vector   vstore;


  /*
   *
   * Test ordinary behavior.
   *
   */


  CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( vector->data, 0, length*sizeof( REAL4 ) );

  DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  CreateVector ( &status, &vector, 0 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  DestroyVector ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  CreateVector ( &status, NULL, length );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  vector = &vstore;
  CreateVector ( &status, &vector, length );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  DestroyVector ( &status, &vector );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of CreateVector and DestroyVector \n" );
          
  return;
}


int main( int argc, char *argv[] )
{
  ParseOptions( argc, argv );

  VectorFactoriesTest();
  ZVectorFactoriesTest();
  CVectorFactoriesTest();
  DVectorFactoriesTest();
  SVectorFactoriesTest();
  I2VectorFactoriesTest();
  I4VectorFactoriesTest();
  I8VectorFactoriesTest();
  U2VectorFactoriesTest();
  U4VectorFactoriesTest();
  U8VectorFactoriesTest();
  CHARVectorFactoriesTest();

  return 0;
}


/*
 * TestStatus ()
 *
 * Routine to check that the status code status->statusCode agrees with one of
 * the codes specified in the space-delimited string ignored; if not,
 * exit to the system with code exitcode.
 *
 */
static void
TestStatus (Status *status, const char *ignored, int exitcode)
{
  char  str[64];
  char *tok;

  if (verbose)
  {
    REPORTSTATUS (status);
  }

  if (strncpy (str, ignored, sizeof (str)))
  {
    if ((tok = strtok (str, " ")))
    {
      do
      {
        if (status->statusCode == atoi (tok))
        {
          return;
        }
      }
      while ((tok = strtok (NULL, " ")));
    }
    else
    {
      if (status->statusCode == atoi (tok))
      {
        return;
      }
    }
  }

  fprintf (stderr, "\nExiting to system with code %d\n", exitcode);
  exit (exitcode);
}


/*
 *
 * ClearStatus ()
 *
 * Recursively applies DETATCHSTATUSPTR() to status structure to destroy
 * linked list of statuses.
 *
 */
void
ClearStatus (Status *status)
{
  if (status->statusPtr)
  {
    ClearStatus      (status->statusPtr);
    DETATCHSTATUSPTR (status);
  }
}


/*
 * Usage ()
 *
 * Prints a usage message for program program and exits with code exitcode.
 *
 */
static void
Usage (const char *program, int exitcode)
{
  fprintf (stderr, "Usage: %s [options]\n", program);
  fprintf (stderr, "Options:\n");
  fprintf (stderr, "  -h         print this message\n");
  fprintf (stderr, "  -q         quiet: run silently\n");
  fprintf (stderr, "  -v         verbose: print extra information\n");
  fprintf (stderr, "  -d level   set debuglevel to level\n");
  exit (exitcode);
}


/*
 * ParseOptions ()
 *
 * Parses the argc - 1 option strings in argv[].
 *
 */
static void
ParseOptions (int argc, char *argv[])
{
  while (1)
  {
    int c = -1;

    c = getopt (argc, argv, "hqvd:");
    if (c == -1)
    {
      break;
    }

    switch (c)
    {
      case 'd': /* set debug level */
        debuglevel = atoi (optarg);
        break;

      case 'v': /* verbose */
        ++verbose;
        break;

      case 'q': /* quiet: run silently (ignore error messages) */
        freopen ("/dev/null", "w", stderr);
        freopen ("/dev/null", "w", stdout);
        break;

      case 'h':
        Usage (argv[0], 0);
        break;

      default:
        Usage (argv[0], 1);
    }

  }

  if (optind < argc)
  {
    Usage (argv[0], 1);
  }

  return;
}

