/*----------------------------------------------------------------------- 
 * 
 * File Name: FindChirpMain.c
 *
 * Author: Allen, B., Brown, D., and Creighton, J. D. E.
 * 
 * Revision: $Id: FindChirpMain.c,v 1.2 2000/04/17 23:27:13 jolien Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include <stdio.h>
#include "mpi.h"
#include "LALStdlib.h"
#include "Comm.h"

NRCSID (MAIN, "$Id: FindChirpMain.c,v 1.2 2000/04/17 23:27:13 jolien Exp $");

void
Master (Status *, MPIId);

void
Slave (Status *, MPIId);

/* global variables */
#define FINDCHIRPGLOBAL_INIT
#include "FindChirpGlobal.h"
#undef FINDCHIRPGLOBAL_INIT

int debuglevel = 1;

int
main (int argc, char *argv[])
{
  static Status  stat;
  MPIDebugParams debug;
  MPIId          id;

  MPI_Init (&argc, &argv);
  MPI_Comm_size (MPI_COMM_WORLD, &id.numProcs);
  MPI_Comm_rank (MPI_COMM_WORLD, &id.myId);
  MPI_Get_processor_name (id.procName, &id.nameLen);

  /* write kill script */
  MPIKillScript (&stat, &id);

  /* debugging example */
  if (0)
  {
    MPIExportEnvironment (&stat, "DISPLAY", id.myId);
    debug.debugger = id.myId == 0 ? "ddd" : NULL ; /* attatch ddd to master */
    debug.progName = argv[0];
    debug.delay    = 15;
    debug.myId     = id.myId;
    MPIDebug (&stat, &debug);
  }

  if (id.myId == 0)
  {
    Master (&stat, id);
    REPORTSTATUS (&stat);
  }
  else
  {
    char fname[64];
    sprintf (fname, "Slave%03d.err", id.myId);
    freopen (fname, "w", stderr);
    sprintf (fname, "Slave%03d.out", id.myId);
    freopen (fname, "w", stdout);
    Slave (&stat, id);
    REPORTSTATUS (&stat);
  }

  MPI_Finalize ();

  LALCheckMemoryLeaks ();
  return 0;
}

